//  PROGRAM : SFCW - DATABASE VIEW
// COPYRIGHT (C) 1999-2009 - ZAC MICRO SYSTEMS
// MODIFIED : 15 OCT 2010
//=============================================================================

#include "DCDIALOG.CH"
#include "set.ch"
#include "xbp.ch"
#include "appevent.ch"
#include 'directry.ch'
#include "DCPICK.CH"
#include "DCAPP.CH"
#include "DCBITMAP.CH"
#include "DCICON.CH"
#include "DCXTOC.CH"
#include "DCCURSOR.CH"
#include "DBSTRUCT.CH"
#include "common.ch"
#include 'cdxdbe.ch'
#include "fileio.ch"
#include "DCDIR.CH"
#include "DCPRINT.CH"

//=============================================================================

#define _RECLOCK          20
#define X_BORDER          OUTLINE BORDERCOLOR 1
#define VK_NUMLOCK        0x90
#define VK_SCROLL         0x91
#define VK_CAPITAL        0x14
#define APPKEY_PRESSED    2
#define CRLF              chr(13)+chr(10)

#define X_RED     1, 3
#define X_NORM    1,15
#define X_YELLOW  1, 7
#define X_GREEN   1, 5
#define X_BLUE    0, 2

#define K_RED     { X_RED    }
#define K_NORM    { X_NORM   }
#define K_YELLOW  { X_YELLOW }
#define K_GREEN   { X_GREEN  }
#define K_BLUE    { X_BLUE   }
//-
//-- DBE_XXXXXXX
//-
#define _DNAME  asStruct[i, DBS_NAME]
#define _DTYPE  asStruct[i, DBS_TYPE]
#define _DLEN   asStruct[i, DBS_LEN ]
#define _DDEC   asStruct[i, DBS_DEC ]

//=============================================================================

static asTags               // Single dim array of index orders
static asStruct             // Structure of selected database
static asPict               // Picture of selected database structure
static apPres               // DCBROWSE parameters
static csFileSpec           // Full FileSpec of selected file
static csPath               // Location of selected file
static csDBF                // Selected FileName with NO Extenstion
static csIDX                // Index extension
static csMEM                // Memo Extenstion
static csMemo               // Memo Contents
static csTitle
static lsEdit               // TRUE if in edit mode
static lsDescend            // TRUE if DESCENDING ORDER
static csSeek               // Seek data
static csDescend            // How is the order displayed
static csOrder, osOrder     // Index order name
static csDbe  , osDbe, osQuery, csName, osName, csQuery
static bsActBlock, lsMDI, osDialog, osDlg
//static csDescend, csUnique


#pragma library( "DCLIPX.LIB" )
#pragma library( "DCLIP1.LIB" )


//=============================================================================
// FUNCTION : MAIN()
//  PURPOSE : Generic Database Browse EDIT
//   CALLED : NONE
//    CALLS : SFCW()
//   IMPORT : [ cData ] [ cCaller ]
//   EXPORT : NONE
//   UPDATE : 15 AUG 09
//=============================================================================

procedure Main( cData, cCaller )

  local cBgn := "SFCW - ", cEnd := " (C) Copyright 2009 ZAC Micro Systems"

//  if DC_IsAppRunning("XBPDIALOG", "IS SFCW RUNNING", "SFCW.EXE", .T.)
//    quit
//  endif
  _LoadDes()
  csPath  := iif( ( cData = NIL ),                              ;
                CurDrive()+"\:"+CurDir(CurDrive())+"\",         ;
                cData)
  csTitle := iif( ( cCaller = NIL ),                             ;
                cBgn+" "+(csPath)+cEnd,                         ;
                cBgn+trim(cCaller)+"  -  "+(csPath)+"  -  "+cEnd)
  lsDescend := .F.
  csDescend := "  NORMAL  "
  _Sfcw()
  return


//-----------------------------------------------------------------------------
// FUNCTION : _SFCW()
//  PURPOSE : Main Screen Menu
//   CALLED : Main()
//    CALLS : DbfView(), DisplayDbe()
//   IMPORT : NONE
//   EXPORT : NONE
//-----------------------------------------------------------------------------

static procedure _Sfcw()

  local getlist := {}, GetOptions, xCaption, nEvent, mp1, mp2, oXbp
  local oMenuBar, oMenuFile, oMenuIdx, oAppWindow := SetAppWindow()
  local cOrgPath := DC_Path(), cExe := appname( .T. )

  DC_Gui( .T. )
  set deleted OFF
  set century ON
  set date format to "mm/dd/yyyy"
  dbesetdefault( "FOXCDX" )
  csDbe   := "FOXCDX"
  csIDX   := ".CDX"
  csOrder := padr( "REC ORDER", 10 )
  //-
  //-- Set Pulldown arrow ICON
  //-
  if "WINDOWSNT5.0" $ upper(strtran( os(), ' ', '' ) )
    xCaption := BITMAP_PICKLIST
  else
    xCaption := chr(31)
  endif
  DC_GetPopUpCaption( {"10.Marlett", "u" } )
  //-
  //-- MENU
  //-
  @02,30 DCPUSHBUTTON CAPTION "VIEW DATABASE"   STATIC COLOR 1,5  SIZE 40,1.7 ;
      ACTION {|| _DbfView() }
  @04,30 DCPUSHBUTTON CAPTION "EXIT"            STATIC COLOR 1,3  SIZE 40,1.7 ;
      ACTION {|| dbcloseall(), DC_ReadGuiEvent(DCGUI_EXIT_OK, getlist) }
  //
  @08,30 DCSAY "SFCW IS LOCATED AT"         SIZE 25             COLOR 2
  @08,47 DCSAY cExe                         SIZE len(cExe),.8   COLOR 0,2
  //
  @10,33 DCSAY    "YOUR DATA IS AT"         SIZE 30             COLOR 2
  @10,47 DCSAY csPath                       SIZE len(csPath),.8 COLOR 0,2
  //
  @12,40 DCSAY "VERSION DATE - 15 OCT 2010" SIZE 23,.8          COLOR 1,5
  //
  DCGETOPTIONS WINDOWWIDTH 800 WINDOWHEIGHT 600 EVAL {|o| setappwindow(o) }   ;
    NOMINBUTTON NOMAXBUTTON NORESIZE TOOLTIPCOLOR GRA_CLR_BLACK, GRA_CLR_YELLOW
  //
  DCREAD GUI TITLE csTitle OPTIONS GetOptions PARENT @osDialog
  //
  DC_ChDir(cOrgPath)
  //-
  //-- Remove Query Temp Files
  //-
  ferase("DCFIELD1.CDX")
  ferase("DCFIELD2.CDX")
  ferase("DCFIELDS.DBF")
  ferase("DCFIELDS.DBT")
  ferase("DCQUERY.DBF")
  ferase("DCQUERY.DBT")
  setappwindow( oAppWindow )
  return


//-----------------------------------------------------------------------------
// FUNCTION : _SFCVIEW()
//  PURPOSE : Create a Browse view of database specified
//   CALLED :
//    CALLS : _BuildIdx(), _SfcSeek(), _SelIdx(), _ShowSfc(), _PutData()
//   IMPORT : NONE
//   EXPORT : NIL
//   UPDATE : 14 OCT 2010
//-----------------------------------------------------------------------------

static procedure _SfcView()

  local getlist := {}, GetOptions, i, lOk := .F., xVal, nGetNumb, aPres
  local cTagName, cIdxName, nArea := select(), cFName, nKeyOrd, bSort, bEval
  local oBrow, oBox, oBar, bFld, nWidth, cPict, cTitle
  local xNIL    := NIL, cTestAlias

  aPres := { {XBP_PP_COL_HA_FGCLR,        GRA_CLR_WHITE    }, ; // Hdg FG Color
             {XBP_PP_COL_HA_BGCLR,        GRA_CLR_DARKGRAY }, ; // Hdg BG Color
             {XBP_PP_COL_DA_FGCLR,        GRA_CLR_BLACK    }, ; // Col FG Color
             {XBP_PP_COL_DA_BGCLR,        GRA_CLR_PALEGRAY }, ; // Col BG Color
             {XBP_PP_COL_DA_HILITE_FGCLR, GRA_CLR_BLACK    }, ; // Col Sel FG
             {XBP_PP_COL_DA_HILITE_BGCLR, GRA_CLR_WHITE    }, ; // Col Sel BG
             {XBP_PP_COL_DA_ROWSEPARATOR, XBPCOL_SEP_DOTTED}, ;
             {XBP_PP_COL_DA_COLSEPARATOR, XBPCOL_SEP_DOTTED}, ;
             {XBP_PP_COL_DA_COMPOUNDNAME, "8.Courier New"  }, ; // Data Font
             {XBP_PP_COL_DA_ROWHEIGHT,    10               }  ; // Data Height
           }
  //
  csOrder   := padr("REC ORDER",10)
  csSeek    := space(20)
  csName    := "GOTO"
  asTags    := {}
  asPict    := {}
  lsEdit    := .F.
  cTitle    := "SFCW Browse - 28 APR 2009 "
  //
  if !empty( csDBF )
    select( 15 )
    cTestAlias := alias()
    dbclosearea()
    //-
    //-- Set the database default directory
    //-
    DC_SetDefault( csPath )
    //-
    //-- Open selected database in shared mode
    //-
    use ( csDBF ) alias ( csDBF ) via ( csDbe ) new shared  // exclusive
    //
    if !neterr()
      //-
      //-- If NO network error occured
      //-
      select ( csDBF )
      //-
      //-- Gather the available indexes
      //-
      _BuildIdx()
      //
    else
      //
      DCMSGBOX "Could NOT open "+csDBF+" SHARED" TIMEOUT 3
      return
      //
    endif
    //-
    //-- Add the database name we are looking at to the title
    //-
    cTitle   += ": VIEWING : "+trim(csPath)+trim(csDBF)+".DBF"
    asStruct := dbstruct()    // GET DATABASE STRUCTURE
    //
    if len(asTags) > 0
      ordsetfocus( 0 )
      //-
      //-- Browse with Index available
      //-
      @.5,01 DCSAY "Seek"                 SIZE  6
      @.5,07 DCGET csSeek PICTURE repl('!',20) GETID "SEARCH"                 ;
          GETEVAL {|oXbp| oXbp:keyboard := {|a,b,o| _SfcSeek(a,b,o, oBrow) }}
      @.5,45 DCSAY "CLICK ON BLUE HEADER TO SORT" SIZE 30 COLOR 2
      @26,45 DCSAY "IDX"                  SIZE  6,.8   COLOR X_YELLOW
      @26,50 DCSAY {|| csOrder }          SIZE 16,.8   COLOR X_YELLOW
    else
      @.5,10 DCSAY "NO INDEX AVAILABLE"   SIZE 30
    endif
    _SetDescend()
    dbgotop()
    //
    @.5,98 DCSAY "RECORDS " + str( reccount(), 6 )   SIZE 20 COLOR 2
    //
    @02,.5 DCBROWSE oBrow DATA csDBF SIZE 112,23.7  PRESENTATION aPres        ;
        FREEZELEFT { 1 }                                                      ;
        EVAL {|o| o:itemselected := {|| _ActOnSfc(90, oBrow, getlist),        ;
                  oBrow:refreshall() } }    FONT "8.Courier New"
    //
    DCSETPARENT TO oBrow
    DCBROWSECOL HEADER "RECNO" DATA {|| str(recno(),6) } WIDTH 6
    //-
    //-- Create Pictures statements from the database structure
    //-
    for i := 1 to len( asStruct )
      do case
        case asStruct[i,DBS_TYPE] = 'M'
          cPict := "XXXXXX"
          DCBROWSECOL DATA                                                    ;
              &('{|| iif(empty('+asStruct[i,DBS_NAME]+'),"<memo>","<MEMO>")}');
              HEADER asStruct[i,DBS_NAME] PICTURE 'XXXXXX' WIDTH 6
        case asStruct[i,DBS_TYPE] = 'L'
          cPict := "XXX"
          DCBROWSECOL DATA &('{|| iif('+asStruct[i,DBS_NAME]+',"YES","NO")}') ;
              HEADER asStruct[i,DBS_NAME] PICTURE cPict WIDTH 3               ;
              COLOR {|| iif(deleted(), { 1, 3}, { 1, 15} ) }
        otherwise
          do case
            case asStruct[i,DBS_TYPE] = 'D'
              cPict := "99/99/9999"
            case asStruct[i,DBS_TYPE] = 'C'
              cPict := repl('!', max(len(asStruct[i, DBS_NAME])+2,  ;
                                 asStruct[i,DBS_LEN]) )
            case asStruct[i,DBS_TYPE] = 'N'
              cPict := repl('9', asStruct[i,DBS_LEN]-asStruct[i,DBS_DEC]) +   ;
                iif(asStruct[i,DBS_DEC] = 0, "",                              ;
                    '.' + repl('9', asStruct[i,DBS_DEC]) )
          endcase
          cFName  := asStruct[i, DBS_NAME]
          nKeyOrd := DC_KeyOrder( cFName )
          if nKeyOrd > 0
            bSort := _SortBlock( nKeyOrd )
            bEval := _Heading()
          else
            bSort := NIL
            bEval := NIL
          endif
          DCBROWSECOL DATA &("{|| "+asStruct[i,DBS_NAME]+"}")                 ;
              HEADER asStruct[i,DBS_NAME] PICTURE cPict                       ;
              WIDTH len( cPict ) SORT bSort EVAL bEval                        ;
              COLOR {|| iif( deleted(), { 1, 3}, { 1, 15} ) }
      endcase
      nGetNumb := len(getlist)
    next
    //
    DCSETPARENT TO
    //
    @26,02 DCSAY "DESCENDING"                             SIZE 12 COLOR 2
    @26,22 DCSAY "UNIQUIE"                                SIZE  9 COLOR 2
    @26,15 DCSAY {|| iif( dbdescend(), "YES", "NO ") }    SIZE  4 COLOR 2
    @26,34 DCSAY {|| iif( DC_IsUnique(),  "YES", "NO ") } SIZE  4 COLOR 2
    @25.7,92 DCPUSHBUTTONXP CAPTION {|| csDescend } SIZE 12,1.2 COLOR 1,5     ;
        ACTION {|| _SetDescend(), DC_GetRefresh(getlist), oBrow:refreshall() };
        HIDE   {|| _WhatOrder() }
    //-
    //-- PUSHBUTTONS
    //-
    @27,01 DCTOOLBAR oBar     SIZE 108,1.2   BUTTONSIZE 9,1.2
    //
    DCSETPARENT TO oBar
    //  1
    DCADDBUTTON CAPTION BITMAP_EXIT_1                   ACCELKEY xbeK_ALT_E   ;
        ACTION {|| DC_ReadGuiEvent(DCGUI_EXIT_OK, getlist) }
    //  2
    DCADDBUTTONXP CAPTION "Top"     SIZE 6              ACCELKEY xbeK_HOME    ;
        OUTLINE BORDERCOLOR 5                                                 ;
        ACTION {|| ( csDBF )->( dbgotop() ),            oBrow:refreshall() }
    //  3
    DCADDBUTTONXP CAPTION "PgUp"    SIZE 6              ACCELKEY xbeK_PGUP    ;
        OUTLINE BORDERCOLOR 5                                                 ;
        ACTION {|| oBrow:pageup(),                      oBrow:refreshall() }
    //  4
    DCADDBUTTONXP CAPTION "PgDn"    SIZE 6              ACCELKEY xbeK_PGUP    ;
        OUTLINE BORDERCOLOR 3                                                 ;
        ACTION {|| oBrow:pagedown(),                    oBrow:refreshall() }
    //  5
    DCADDBUTTONXP CAPTION "End"     SIZE 6              ACCELKEY xbeK_END     ;
        OUTLINE BORDERCOLOR 3                                                 ;
        ACTION {|| ( csDBF )->( dbgobottom() ),         oBrow:refreshall() }
    //  6
    DCADDBUTTON   CAPTION BITMAP_EDIT_1                 ACCELKEY xbeK_ALT_E   ;
        ACTION {|| _ActOnSfc(90, oBrow, getlist),     oBrow:refreshcurrent() }
    //  7
    DCADDBUTTON CAPTION BITMAP_NEW_1                    ACCELKEY xbeK_ALT_N   ;
        ACTION {|| _ActOnSfc(91, oBrow, getlist),     oBrow:refreshall() }
    //  8
    DCADDBUTTONXP CAPTION "Export"          X_BORDER           COLOR X_GREEN  ;
        ACTION {|| _Export() }
    //  9
    DCADDBUTTONXP CAPTION "&Filter"         X_BORDER           COLOR X_YELLOW ;
        ACTION {|| _SFCFilter(),                      oBrow:refreshall() }
    // 10
    DCADDBUTTON CAPTION BITMAP_DELETE_1                 ACCELKEY xbeK_ALT_D   ;
        ACTION {|| _ActOnSfc(92, oBrow, getlist),     oBrow:refreshcurrent() }
    // 11
    DCADDBUTTONXP CAPTION "&PACK"   SIZE 6  X_BORDER           COLOR X_RED    ;
        ACTION {|| DC_Pack(.F.,"S"), DC_GetRefresh(getlist), oBrow:refreshall() }
    // 12
    DCADDBUTTONXP CAPTION "&Structure"      X_BORDER           COLOR X_BLUE   ;
        ACTION {|| _ShowStruct() }
    // 13
    DCADDBUTTONXP CAPTION "&INDEX"          X_BORDER           COLOR 1,4      ;
        ACTION {|| _SelIdx(oBrow), DC_GetRefresh(getlist), oBrow:refreshall() }
    DCSETPARENT TO
    //
    DCGETOPTIONS WINDOWWIDTH 800 WINDOWHEIGHT 600                             ;
        NOMINBUTTON NOMAXBUTTON NORESIZE                                      ;
        TOOLTIPCOLOR GRA_CLR_BLACK, GRA_CLR_YELLOW
    //
    DCREAD GUI TITLE cTitle OPTIONS GetOptions EVAL {|o| setappwindow(o) }
    //
    dbclosearea()
    if len( cTestAlias ) # 0
      use ( cTestAlias )
      if !netErr()
        set index to ( cTestAlias )
      endif
    endif
    select( nArea )
  endif
  return


//-----------------------------------------------------------------------------
// FUNCTION : _WHATORDER()
//  PURPOSE : Determine if a index order has been set
//   CALLED : _SFCVIEW()
//    CALLS : NONE
//   IMPORT : NONE
//   EXPORT : TRUE if Natural Order
//   UPDATE : 15 JUL 09
//-----------------------------------------------------------------------------

static function _WhatOrder()

  local nOrd := ordnumber()
  return ( nOrd = 0 )


//-----------------------------------------------------------------------------
// FUNCTION : _SETDESCEND()
//  PURPOSE : Set INDEX Order
//   CALLED : _SFCVIEW()
//    CALLS : NONE
//   IMPORT : NONE
//   EXPORT : DESCENDING or NORMAL
//   UPDATE : 15 JUL 09
//-----------------------------------------------------------------------------

static procedure _SetDescend()

  if lsDescend
    csDescend := "  NORMAL  "
    dbsetdescend( .T. )
  else
    csDescend := "DESCENDING"
    dbsetdescend( .F. )
  endif
  lsDescend := !lsDescend
  dbgotop()
  return


//-----------------------------------------------------------------------------
// FUNCTION : _SORTBLOCK()
//  PURPOSE : Set INDEX Order
//   CALLED : _SFCVIEW()
//    CALLS : NONE
//   IMPORT : NONE
//   EXPORT : IDX ORDER
//   UPDATE : 23 MAR 09
//-----------------------------------------------------------------------------

static function _SortBlock( nKeyOrd )

  return {|| ( csDBF )->( ordsetfocus( nKeyOrd ) ) }


//-----------------------------------------------------------------------------
// FUNCTION : _HEADING()
//  PURPOSE : Set SORT FONT
//   CALLED : _SFCVIEW()
//    CALLS : NONE
//   IMPORT : NONE
//   EXPORT : IDX ORDER
//   UPDATE : 23 MAR 09
//-----------------------------------------------------------------------------

static function _Heading()

  return {|o| o:heading:setfontcompoundname("8.Lucida Bright Italic.BOLD")}


//-----------------------------------------------------------------------------
// FUNCTION : _SHOWSTRUCT()
//  PURPOSE : Show Structure of Database
//   CALLED : _SFCVIEW()
//    CALLS : NONE
//   IMPORT : NONE
//   EXPORT : NONE
//   UPDATE :  5 MAY 09
//-----------------------------------------------------------------------------

static procedure _ShowStruct()

  local getlist := {}, GetOptions, oBrow, oBar, aTemp, cAlias, lOk := .F.

  aTemp  := dbstruct()
  cAlias := alias()
  //
  @00,00 DCSAY ""
  @.2,01 DCSAY "STRUCTURE OF ( "+trim(cAlias)+" )"  SAYSIZE 30
  @.2,30 DCSAY "FLDS "+trim( str( len(aTemp),3,0) ) SAYSIZE 10
  //
  @01,01 DCBROWSE oBrow DATA aTemp  PRESENTATION apPres   SIZE 54.3,21      ;
      NOHSCROLL NOSIZECOLS
  //
  DCSETPARENT TO oBrow
  DCBROWSECOL HEADER "FNAME"  WIDTH 15  ELEMENT 1
  DCBROWSECOL HEADER "FTYPE"  WIDTH  5  ELEMENT 2 PICTURE "!!"
  DCBROWSECOL HEADER "FLEN"   WIDTH  5  ELEMENT 3 PICTURE "9999"
  DCBROWSECOL HEADER "FDEC"   WIDTH  5  ELEMENT 4 PICTURE "9999"
  DCSETPARENT TO
  //
  @23,01 DCTOOLBAR oBar SIZE 27,1.2 BUTTONSIZE 9,1.2
  //
  DCSETPARENT TO oBar
  DCADDBUTTON CAPTION BITMAP_EXIT_1                       ACCELKEY xbeK_ALT_E ;
      ACTION {|| DC_ReadGuiEvent(DCGUI_EXIT_OK, getlist) }
  DCADDBUTTONXP CAPTION "PgUp"                            ACCELKEY xbeK_PGUP  ;
      ACTION {|| oBrow:pageup(),      oBrow:refreshall() }
  DCADDBUTTONXP CAPTION "PgDn"                            ACCELKEY xbeK_PGDN  ;
      ACTION {|| oBrow:pagedown(),    oBrow:refreshall() }
  DCSETPARENT TO
  //
  DCGETOPTIONS NOMINBUTTON NOMAXBUTTON NORESIZE
  //
  DCREAD GUI TITLE "STRUCTURE OF "+cAlias+".DBF" OPTIONS GetOptions FIT MODAL ;
      TO lOK    EVAL {|o| setappwindow(o) }
  //
  return


//-----------------------------------------------------------------------------
// FUNCTION : _SFCFILTER()
//  PURPOSE : Setup a query filter
//   CALLED : _SFCVIEW()
//    CALLS : NONE
//   IMPORT : NONE
//   EXPORT : NONE
//-----------------------------------------------------------------------------

static procedure _SFCFilter()

  local bQuery, cQuery := ""

  cQuery := DC_QueryBuild( cQuery )
  if !empty( cQuery )
    bQuery := &( '{||' + cQuery + '}' )
    dbsetfilter( bQuery, cQuery )
  elseif !empty( dbfilter() )
    dbclearfilter()
  endif
  return


//-----------------------------------------------------------------------------
// FUNCTION : _PUTDATA()
//  PURPOSE : Save field edited using VALID statement
//   CALLED : _SFCVIEW()
//    CALLS : NONE
//   IMPORT : oGet
//   EXPORT : TRUE
//-----------------------------------------------------------------------------
/***
static function _PutData(oGet)

  local aCargo   := oGet:cargo[3]

  if rlock()
    fieldput( fieldpos(aCargo[2]), oGet:getdata() )
    dbcommit()
    dbunlock()
  endif
  return (.T.)
***/


//-----------------------------------------------------------------------------
// FUNCTION : _ACTONSFC()
//  PURPOSE : Actions to perform on browse
//   CALLED : _SFCVIEW(),
//    CALLS : NONE
//   IMPORT : nAction - What to do
//            oBrow   - Browse object
//            aGets   - GetList
//   EXPORT : TRUE
//   UPDATE : 15 OCT 2010
//-----------------------------------------------------------------------------

static function _ActOnSfc(nAction, oBrow, aGets)

  local nSel, cText, cOrder, aData, i, nRec := recno(), lOk, nArea, nNowRec

  if ( nAction < 90 ) .or. ( nAction > 92 )
    // INVALID NUMBER CALL - NUMBERS USED 90, 91, 92
    return ( .F. )
  endif
  //-
  //-- BUTTON - EDIT
  //-
  if nAction = 90
    if _EditRec(oBrow, aGets, @aData, 1, nRec)// IF EDIT HIGHLITED REDORD
      if valtype( aData ) == 'A'              //  IF WE HAVE A ARRAY
        if DC_RecLock( _RECLOCK )             //   IF GOOD RECORD LOCK
          for i := 1 to len( aData )          //    FOR 1 TO LEN OR ARRAY
            if asStruct[i, DBS_TYPE] = 'M'    //     IF FIELD IS MEMO
              aData[ i ] := csMemo            //      RESET MEMO FIELD
            endif                             //     ENDIF
          next                                //    NEXT
          gather( aData )                     //    WRITE RECORD
          dbunlock()                          //    UNLOCK
          dbcommit()                          //    COMMIT
          DC_GetRefresh(aGets)                //   REFRESH GETS ON DISPLAY
          DC_GetOrigSet(aGets)                //   RESET GETS FOR DISPLAY
        endif                                 //   ENDIF
      endif                                   //  ENDIF
    else                                      // ELSE
      DC_GetRefresh(aGets)                    //   REFRESH GETS ON DISPLAY
      DC_GetOrigSet(aGets)                    //   RESET GETS FOR DISPLAY
    endif                                     // ENDIF
    return ( .T. )                            // RETURN
    //
  elseif nAction = 91
    //-
    //-- BUTTON - INSERT / NEW RECORD
    //-
    nRec := 0
    if _EditRec(oBrow, aGets, @aData, 2, nRec)// IF CREATE NEW RECORD
      if valtype( aData ) == 'A'              //  IF WE HAVE A ARRAY
        if DC_AddRec( 20 )                    //   IF ADDING RECORD
          if DC_RecLock( _RECLOCK )           //    IF RECORD LOCKED
            for i := 1 to len( aData )        //     FOR 1 TO LEN OF ARRAY
              if asStruct[i, DBS_TYPE] = 'M'  //      IF MEMO FIELD
                aData[ i ] := csMemo          //       RESET MEMO DATA
              endif                           //      ENDIF
            next                              //     NEXT
            gather( aData )                   //     WRITE RECORD
            dbunlock()                        //     UNLOCK
            dbcommit()                        //     COMMIT
          endif                               //    ENDIF
        endif                                 //   ENDIF
        DC_GetRefresh(aGets)                  //   REFRESH GETS ON DISPLAY
        DC_GetOrigSet(aGets)                  //   RESET GETS FOR DISPLAY
      endif                                   //  ENDIF
    else                                      // ELSE
      dbgoto( nRec )                          //  GOTO ORG REC
      DC_GetRefresh(aGets)                    //   REFRESH GETS ON DISPLAY
      DC_GetOrigSet(aGets)                    //   RESET GETS FOR DISPLAY
    endif                                     // ENDIF
    return ( .T. )                            // RETURN
    //
  elseif nAction = 92
    //-
    //-- BUTTON - DELETE
    //-
    cText := iif(deleted(), "RESTORE", "DELETE")
    if DC_RecLock( _RECLOCK )                 // IF RECORD LOCKED
      if deleted()                            //  IF DELETED
        dbrecall()                            //   REMOVE DELETED STATUS
      else                                    //  ELSE
        dbdelete()                            //   MARK AS DELETED
      endif                                   //  ENDIF
      dbunlock()                              //  UNLOCK RECORD
      dbcommit()                              //  COMMIT
    endif                                     // ENDIF
    DC_GetRefresh(aGets)                      // REFRESH GETS ON DISPLAY
    DC_GetOrigSet(aGets)                      // RESET GET FOR DISPLAY
    return ( .T. )                            // RETURN
  endif                                       // ENDIF
  return ( .T. )                              // RETURN


//-----------------------------------------------------------------------------
// FUNCTION : _EDITREC()
//  PURPOSE : Edit Current record data
//   CALLED : _Action()
//    CALLS : NONE
//   IMPORT : aBrow   - Current Record
//            aGets   - Curent Getlist
//            aData   - Current structure data
//            nMode   - Action 1 == Current, 2 == NEW
//            cText   - Action Text
//            nRec    - Edit/Add Record number
//   EXPORT : NONE
//   UPDATE : 15 OCT 2010
//-----------------------------------------------------------------------------

function _EditRec(oBrow, aGets, aData, nMode, nRec)

  local getlist := {}, oBar, i, lOk := .F., oOrg := SetAppWindow()
  local nCol := 01, nRow := 01, cData := "", GetOptions, aStruct, cTitle
  local cText := " Record", cFont := "8.Courier New", cPict, nOrgRec

  DEFAULT nMode := 1        // 1 = Current, 2 = New
  //
  cTitle  := iif( nMode = 1, "EDIT", "NEW") + cText
  nOrgRec := recno()
  dbgoto( nRec )
  aStruct := dbstruct()
  aData   := array( fcount() )
  csMemo  := ""
  if nMode = 1
    aData := scatter()                      // READ HILITED RECORD
  else
    for i := 1 to len(aData)                // CREATE NEW EMPTY RECORD IN aData
      if aStruct[i, DBS_TYPE]     == 'M'    // IF MEMO FIELD
        aData[i] := ""
      elseif aStruct[i, DBS_TYPE] == 'D'    // ELSEIF DATE FIELD
        aData[i] := date()
      elseif aStruct[i, DBS_TYPE] == 'C'    // ELSEIF CHARACTER FIELD
        aData[i] := space(aStruct[i, DBS_LEN])
      elseif aStruct[i, DBS_TYPE] == 'L'    // ELSEIF LOGICAL FIELD
        aData[i] := .F.
      elseif aStruct[i, DBS_TYPE] == 'N'    // ELSEIF NUMERIC FIELD
        aData[i] := 0
      endif
    next
  endif
  //-
  //-- CREATE DISPLAY GETS AND PICTURES FOR EACH FIELD IN THE DATABASE
  //-
  for i := 1 to len( aData )                  // Create display gets/pictures
    if (nCol + aStruct[i, DBS_LEN]) > 90
      nRow++
      nCol  := 01
    endif
    //-
    //-- SET PICTURE FOR EACH TYPE
    //-
    if      aStruct[i, DBS_TYPE] = 'D'        // IF DATE
      cPict := "99/99/9999"
    elseif  aStruct[i, DBS_TYPE] = 'C'        // ELSEIF CHARACTER
      cPict := repl('!', aStruct[i, DBS_LEN])
    elseif  aStruct[i, DBS_TYPE] = 'N'        // ELSEIF NUMERIC
      cPict := repl('9', aStruct[i, DBS_LEN]) +                 ;
                iif( !empty(aStruct[i, DBS_DEC] ),              ;
                     '.' + repl('9', aStruct[i, DBS_DEC]), "" )
    endif
    //-
    //-- SETUP THE SCREEN POSITION OF THE FIELD TO DISPLAY
    //-
    if aStruct[i, DBS_TYPE] = 'M'             // IF MEMO FIELD
      csMemo := aData[ i ]    // fieldget( fieldpos( "MEMO" ) )
      nCol   := 01
      nRow   += 02
      @nRow,nCol DCMULTILINE csMemo SIZE 110,26-nRow FONT '8.Terminal'
      nRow   += 02
    elseif aStruct[i, DBS_TYPE] = 'L'         // ELSEIF LOGICAL
      @nRow,nCol-.3 DCCHECKBOX aData[ i ] PROMPT aStruct[i, DBS_NAME]   ;
          FONT cFont  COLOR 2,7
      nCol   += len(aStruct[i, DBS_NAME]) + 6
    else                                      // ELSE ALL OTHER FIELDS
      @nRow,nCol DCSAY aStruct[i, DBS_NAME] SIZE len( aStruct[i, DBS_NAME])+2 ;
          COLOR 2,7
      @nRow,nCol+len(aStruct[i,DBS_NAME])+2 DCGET aData[ i ] PICT cPict
      nCol   += len(cPict) + 17
    endif
  next
  //
  @27,02 DCTOOLBAR oBar SIZE 18,1.2 BUTTONSIZE 9,1.2
  //
  DCSETPARENT TO oBar
  DCADDBUTTONXP CAPTION "SAVE"                                  COLOR 1,5    ;
      ACTION {|| lOk := .T., DC_ReadGuiEvent(DCGUI_EXIT_OK, getlist) }
  DCADDBUTTON CAPTION BITMAP_CANCEL_1                     ACCELKEY xbeK_ALT_C;
      ACTION {|| lOk := .F., DC_ReadGuiEvent(DCGUI_EXIT_ABORT, getlist) }
  DCSETPARENT TO
  //
  DCGETOPTIONS WINDOWWIDTH 800 WINDOWHEIGHT 600                               ;
      TABSTOP HILITEGETS GRA_CLR_RED                                          ;
      SAYFONT cFont GETFONT cFont
  //
  DCREAD GUI TITLE cTitle OPTIONS GetOptions MODAL TO lOk                      ;
      EVAL {|o| setappwindow(o) }
  //
  if lOk
    DCMSGBOX "SAVE RECORD"            TIMEOUT 2
    dbgoto( nRec )
  elseif !lOk
    DCMSGBOX "DON'T SAVE GO ORG REC"  TIMEOUT 2
    dbgoto( nOrgRec )
  endif
  SetAppWindow(oOrg)
  return ( lOk )    // _ACTONSFC() WILL DETERMINE SAVE OR CANCEL OF THE EDIT


//-----------------------------------------------------------------------------
// FUNCTION : _SELIDX()
//  PURPOSE : Select index for database
//   IMPORT : oBr ( oBrow )
//   CALLED :
//    CALLS : NONE
//   EXPORT : NONE
//   UPDATE : 10 JUL 09
//-----------------------------------------------------------------------------

static procedure _SelIdx( oBr )

  local getlist := {}, GetOptions, oBrow, lOk := .F., nSel := 0, nRow, aPres

  if !empty( asTags )                       // IF WE HAVE ANY TAGS?
    //-
    //-- HAVE ONE OR MORE IDX TAG NAMES
    //-
    if len(asTags) = 1
      ordsetfocus( 0 )                      // DEFAULT TO RECORD ORDER
      csOrder := padr( "REC ORDER", 10 )
    else
      //-
      //-- HAVE MORE THAN ONE TAG NAME
      //-
      @01,01 DCBROWSE oBrow DATA asTags PRESENTATION apPres SIZE 21.9,19.6  ;
          NOHSCROLL NOSIZECOLS                                              ;
          EVAL {|o| o:itemselected := {|| lOk := .T.,                       ;
                    DC_ReadGuiEvent(DCGUI_EXIT_OK, getlist) } }
      //
      DCBROWSECOL HEADER "INDEX" WIDTH 20 ELEMENT 1 PARENT oBrow
      //
      DCGETOPTIONS NOMINBUTTON NOMAXBUTTON NORESIZE
      //
      DCREAD GUI TITLE "SELECT INDEX" OPTIONS GetOptions ADDBUTTONS FIT     ;
          MODAL TO lOk    EVAL {|o| setappwindow(o) }
      //
      if lOk
        nRow    := DC_BrowseRow( oBrow )              // FIND THE SELECTION #
        csOrder := padr( trim( asTags[nRow,1] ), 10 ) // FIND NEW INDEX ORDER
        ordsetfocus( trim( csOrder ) )                // SET NEW INDEX ORDER
      else
        ordsetfocus( 0 )                      // SET FOCUS TO ZERO
        csOrder := padr( "REC ORDER", 10 )    // CANCEL ANY INDEX ORDER
      endif
      dbskip( 0 )
    endif
    oBr:refreshall()                    // REFRESH CALLING BROWSE DISPLAY
  endif
  if valtype(osOrder) == 'O'
    osOrder:setcaption( csOrder )       // SHOW IDX ORDER ON BROWSE SCREEN
  endif
  return


//-----------------------------------------------------------------------------
// FUNCTION : _BUILDIDX()
//  PURPOSE :
//   CALLED :
//    CALLS :
//   IMPORT : cDbf
//   EXPORT : NONE
//-----------------------------------------------------------------------------

procedure _BuildIdx()

  local cTagName, cIdxName, nSel := 1

  cTagName := ""
  cIdxName := alltrim(csDBF)+csIDX    // '.CDX' or '.NTX'
  asTags   := {}
  //-
  //-- If DBASEII file format
  //-
  if csIDX == '.NTX'
    //
  endif
  if !empty( csIDX ) .and. file( csPath + ( csDBF ) + ( csIDX ) )
    set index to (csDBF)
    //-
    //-- Find CDX Tags for index
    //-
    cTagName := ordkey( nSel++ )
    lsEdit   := .F.
    while !empty( cTagName )
      aadd(asTags, { padr( ordname( nSel - 1 ), 20 ) } )
      cTagName := ordkey( nSel++ )
    end
  endif
  return


//-----------------------------------------------------------------------------
// FUNCTION : _SFCSEEK()
//  PURPOSE : Position to character entry
//   CALLED : _SFCVIEW()
//    CALLS : NONE
//   IMPORT : a,b,o
//            oBrow
//   EXPORT : NONE
//-----------------------------------------------------------------------------

static procedure _SfcSeek(a,b,o, oBrow)

  local lClearBuffer := .T., lSoftSeek := set(_SET_SOFTSEEK, .T.)
  local cIdx := ordsetfocus()

  if a == xbeK_UP       ; oBrow:up()
  elseif a = xbeK_DOWN  ; oBrow:down()
  elseif a = xbeK_PGUP  ; oBrow:pageup()
  elseif a = xbeK_PGDN  ; oBrow:pagedown()
  elseif a = xbeK_END   ; oBrow:gobottom()
  elseif a = xbeK_HOME  ; oBrow:gotop()
  else
    if !empty(cIdx)
      if dbseek( upper(alltrim( o:EditBuffer() ) ) )
        lClearBuffer := .F.
      endif
    endif
  endif
  oBrow:refreshall()
  if lClearBuffer
    o:Get:reset()
  endif
  set(_SET_SOFTSEEK, lSoftSeek)
  return


//-----------------------------------------------------------------------------
// FUNCTION : _EXPORT()
//  PURPOSE : Export Select data to text file
//   CALLED :
//    CALLS : NONE
//   IMPORT : NONE
//   EXPORT : NONE
//   UPDATE : 16 APR 09
//-----------------------------------------------------------------------------

static procedure _Export()

  local getlist := {}, GetOptions, lOk := .F., nHdl, cLine, bEval, cMem
  local oProg, oMyBox, nMaxCnt, nCnt := 0, nRec := 0
  local aFlds := {}, xData, i, cFld

  _SelFlds( @aFlds )
  if len(aFlds) = 0
    return
  endif
  //
  csQuery := ""
  if !_Query()
    return
  endif
  //
  if empty( alltrim( csQuery ) )
    bEval := &( "{|| .T. }" )
  else
    bEval := &( "{|| " + alltrim( csQuery ) + "}" )
  endif
  //
  nMaxCnt := reccount()
  dbgotop()
  //
  if !eof()
    //
    @02,05 DCSAY "*** CREATING  "+csDbf+".TXT  FILE ***"
    @03,05 DCPROGRESS oProg SIZE 55,1.5 PERCENT EVERY 1 MAXCOUNT nMaxCnt      ;
        COLOR GRA_CLR_RED
    //
    DCGETOPTIONS NOMINBUTTON NOMAXBUTTON NORESIZE
    //
    DCREAD GUI TITLE "EXPORT TO" OPTIONS GetOptions PARENT @oMyBox FIT EXIT
    oMyBox:show()
    //
    ferase( ( csDbf ) + ".TXT" )
    nHdl := fcreate( ( csDbf ) + ".TXT", FC_NORMAL )
    //-
    //-- Create row one that contains the field names
    //-
    cLine := ""
    for i := 1 to len( aFlds )
      cLine += aFlds[i, 1] + ","
    next
    cLine := substr( cLine, 1, len( cLine ) - 1 ) + CRLF
    //-
    //-- Create rows containing the contents of each field
    //-
    while !eof()
      //
      cFld  := aFlds[1, 1]
      xData := &( cFld )
      //
      if eval( bEval )
        //
        cLine := ""
        //
        for i := 1 to len( aFlds )
          cFld  := aFlds[i, 1]
          xData := &( cFld )
          do case
            case aFlds[i, 2] = 'C'
              cLine += '"'+alltrim(xData)+'",'
            case aFlds[i, 2] = 'N'
              cLine += '"'+alltrim(str(xData,aFlds[i, 3],aFlds[i, 4]))+'",'
            case aFlds[i, 2] = 'L'
              cLine += iif(xData, '"YES"','"NO"')+','
            case aFlds[i, 2] = 'D'
              cLine += '"'+dtoc(xData)+'",'
          endcase
        next
        //
        cLine := substr( cLine, 1, len( cLine ) - 1 ) + CRLF
        fwrite(nHdl, cLine)
        nRec++
        //
      endif
      //
      nCnt++
      DC_GetProgress(oProg,nCnt,nMaxCnt)
      dbskip()
    end
    //
    fclose(nHdl)
    oMyBox:destroy()
    //
    if nRec > 0
      //
      cLine := memoread( csDbf + ".TXT" )
      //
      DCMSGBOX "CREATED "+ltrim(str(nRec,5,0))+" RECORDS REVIEW" YESNO TO lOk
      //
      if lOk
        cMem  := DC_MemoEdit(cLine,,,,,.T.,180,,,,,,"EXPORTED")
        //
        if len( cMem ) > 0
          ferase(   ( csDbf ) + ".BAK")
          frename(  ( csDbf ) + ".TXT", ( csDbf ) + ".BAK")
          memowrit( ( csDbf ) + ".TXT", cMem )
        endif
        //
      endif
      //
    endif
    //
  endif
  return


//-----------------------------------------------------------------------------
// FUNCTION : _SELFLDS()
//  PURPOSE : Select fields for Export to text file
//   CALLED :
//    CALLS : NONE
//   IMPORT : @aFlds
//   EXPORT : NONE
//-----------------------------------------------------------------------------

static procedure _SelFlds(aFlds)

  local aListFld, aPickFld, getlist := {}, GetOptions, oBrow, oDialog, oBox
  local cAlias := select(), i, j, cName, lOk := .F.

  aListFld := array( fcount() )
  aFields( aListFld )
  //
  @01,03 DCPICKLIST aPickFld LIST aListFld CAPTION "Avail Fields", "SELECTED" ;
      SIZE 48,16 DATALINK {|| lOk := .T.,                                     ;
          DC_ReadGuiEvent(DCGUI_EXIT_OK, getlist) }
  //
  DCREAD GUI TITLE "FIELD PICKLIST" MODAL FIT EVAL {|o| SetAppWindow(o) }
  //
  if lOk
    for i := 1 to len(asStruct)
      cName := asStruct[i, 1]
      if (j := ascan(aPickFld, cName)) # 0
        aadd(aFlds, {asStruct[i,DBS_NAME],asStruct[i,DBS_TYPE], ;
                     asStruct[i,DBS_LEN ],asStruct[i,DBS_DEC ] })
      endif
    next
  endif
  return


//-----------------------------------------------------------------------------
// FUNCTION : _QUERY()
//  PURPOSE : Setup Select fields for Export to text file
//   CALLED :
//    CALLS : _SelQFld()
//   IMPORT : NONE
//   EXPORT : NONE
//   UPDATE : 15 APR 09
//-----------------------------------------------------------------------------

static function _Query()

  local getlist := {}, GetOptions, lOk := .F., nSel := 0, oSelect
  local cTitle := "EXPORT QUERY"

  @00,00 DCSAY " "
  @01,01 DCSAY "EXPORT RECORD THAT MATCH THE FOLLOWING" SIZE 45
  @02,01 DCSAY {|| iif(empty(alltrim(csQuery)), "ALL", csQuery) }             ;
      SIZE 70 COLOR GRA_CLR_BLUE OBJECT osQuery
  //
  @04,10 DCPUSHBUTTON CAPTION "EXPORT ALL RECORDS"      SIZE 25,2             ;
      ACTION {|| csQuery := "", oSelect:setcaption( "SELECT A FIELD" ),      ;
                 DC_GetRefresh(getlist) }
  @06,10 DCPUSHBUTTON CAPTION "SELECT A FIELD"          SIZE 25,2             ;
      ACTION {|| _SelQFld(oSelect), DC_GetRefresh(getlist) } OBJECT oSelect
  @09,80 DCSAY " "
  //
  DCGETOPTIONS NOMINBUTTON NOMAXBUTTON NORESIZE EVAL {|o| setappwindow(o) }
  //
  DCREAD GUI TITLE cTitle OPTIONS GetOptions ADDBUTTONS MODAL FIT TO lOk
  //
  return (lOk)


//-----------------------------------------------------------------------------
// FUNCTION : _SELQFLD()
//  PURPOSE : Setup Select fields for Export to text file
//   CALLED :
//    CALLS : _Query()    _SelQAction()
//   IMPORT : oSelect
//   EXPORT : NONE
//   UPDATE : 25 MAR 09
//-----------------------------------------------------------------------------

static procedure _SelQFld(oSelect)

  local getlist := {}, GetOptions, lOk := .F., nSel := 0, oBrow

  @01,01 DCBROWSE oBrow DATA asStruct PRESENTATION apPres SIZE 34.8,17.4      ;
      NOHSCROLL NOSIZECOLS                                                    ;
      EVAL {|o| o:itemselected := {|| lOk := .T.,                             ;
                DC_ReadGuiEvent(DCGUI_EXIT_OK, getlist) } }
  //
  DCBROWSECOL ELEMENT 1 HEADER "FIELD" PICTURE repl('!',20) WIDTH 20          ;
      PARENT oBrow
  //
  DCGETOPTIONS NOMINBUTTON NOMAXBUTTON NORESIZE
  //
  DCREAD GUI TITLE "EQUERY FIELD" OPTIONS GetOptions ADDBUTTONS FIT MODAL TO  ;
      lOk   EVAL {|o| SetAppWindow(o) }
  //
  if lOk
    _SelQAction( DC_BrowseRow(oBrow), oSelect)
  endif
  return


//-----------------------------------------------------------------------------
// FUNCTION : _SELQACTION()
//  PURPOSE : Setup Select fields for Export to text file
//   CALLED : _SelQFld()
//    CALLS : _OperMenu()    _SelAction()
//   IMPORT : oSelect
//   EXPORT : NONE
//   UPDATE : 25 MAR 09
//-----------------------------------------------------------------------------

static procedure _SelQAction(nElem, oSelect)

  local getlist := {}, GetOptions, oBrow, lOk := .F., nSel := 0, cDesc, cSmall
  local cFldName, cType, nLen, nDec, cOper, aMenu, aOperator, cVal := ""
  local cTitle := "OPERATION ON FIELD"

  cFldName  := asStruct[nElem, 1]
  cType     := asStruct[nElem, 2]
  nLen      := asStruct[nElem, 3]
  nDec      := asStruct[nElem, 4]
  //
  _OperMenu(@aMenu, @aOperator, cType)
  //
  if len(aMenu) > 0
    //
    @01,01 DCBROWSE oBrow DATA aMenu    PRESENTATION apPres   SIZE 34.9,17.4  ;
        NOHSCROLL NOSIZECOLS                                                  ;
        EVAL {|o| o:itemselected := {|| lOk := .T.,                           ;
                  DC_ReadGuiEvent(DCGUI_EXIT_OK, getlist) } }
    //
    DCBROWSECOL ELEMENT 1 HEADER "OPERATION" PICTURE repl('!',20) WIDTH 20    ;
        PARENT oBrow
    //
    DCGETOPTIONS NOMINBUTTON NOMAXBUTTON NORESIZE EVAL {|o| setappwindow(o) }
    //
    DCREAD GUI TITLE cTitle OPTIONS GetOptions ADDBUTTONS FIT MODAL TO lOk
    //
    if lOk
      nSel  := DC_BrowseRow(oBrow)
      cOper := aOperator[nSel]
      cDesc := aMenu[nSel,1]
      //
      if _SetAction(cOper, cDesc, cFldName, cType, nLen, nDec, @cVal)
        if !empty( alltrim(csQuery) )
          csQuery += ".and."
        endif
        do case
          case cType = 'N'
            csQuery += (cFldName)+(cOper)+"val("+(cVal)+")"
          case cType = 'D'
            csQuery += "dtos("+(cFldName)+")"+(cOper)+(cVal)
          case cType = 'C'
            if '$' $ cOper
              cSmall  := substr(cVal,2,len(cVal)-2)
              cVal    := '"'+alltrim(cSmall)+'"'
              csQuery += (cVal)+(cOper)+(cFldName)
            else
              csQuery += (cFldName)+(cOper)+(cVal)
            endif
          case cType = 'L'
            if cVal = 'T'
              csQuery += 'iif('+(cFldName)+',"YES","NO")=="YES"'
            else
              csQuery += 'iif('+(cFldName)+',"YES","NO")=="NO"'
            endif
        endcase
        //
        if !empty( alltrim(csQuery) )
          oSelect:setcaption("AND THIS FIELD")
        else
          oSelect:setcaption("SELECT A FIELD")
        endif
        //
      endif
      //
    endif
  endif
  return


//-----------------------------------------------------------------------------
// FUNCTION : _SELACTION()
//  PURPOSE : Setup Select fields for Export to text file
//   CALLED : _SelQAction()
//    CALLS : _OperMenu()    _SelAction()
//   IMPORT : oSelect
//   EXPORT : NONE
//-----------------------------------------------------------------------------

static function _SetAction(cOper, cDesc, cFldName, cType, nLen, nDec, cVal)

  local getlist := {}, GetOptions, lOk := .F., xAction, cAction := ""
  local cPict := repl('9',nLen)+"."+repl('9',nDec), nRow
  local cTitle := "ACTION TO EXCLUDE"

  cVal    := ""
  cAction := cFldName + "  " + cDesc
  nRow    := len(cAction)
  //
  @01,01 DCSAY cAction SAYSIZE nRow+4
  //
  if cType = 'C'
    xAction := space(nLen)
    @01,nRow+2 DCGET xAction PICTURE repl('!',nLen)
  elseif cType = 'N'
    xAction := 0
    @01,nRow+2 DCGET xAction PICTURE cPict
  elseif cType = 'D'
    xAction := date()
    @01,nRow+2 DCGET xAction POPUP {|d| DC_PopDate(d) }
  elseif cType = 'L'
    return (.T.)
  else
    return (lOk)
  endif
  //
  DCGETOPTIONS NOMINBUTTON NOMAXBUTTON NORESIZE EVAL {|o| setappwindow(o) }
  //
  DCREAD GUI TITLE cTitle OPTIONS GetOptions ADDBUTTONS FIT MODAL TO lOk
  //
  if lOk
    do case
      case cType = 'C'
        cVal += '"'+xAction+'"'
      case cType = 'D'
        cVal += '"'+dtos(xAction)+'"'
      case cType = 'N'
        cVal += '"'+alltrim(str(xAction,nLen,nDec))+'"'
      case cType = 'L'
        cVal += '"'+xAction+'"'
    endcase
  endif
  return (lOk)


//-----------------------------------------------------------------------------
// FUNCTION : _OPERMENU()
//  PURPOSE : Select action on field for Export to text file
//   CALLED : _SelAction()
//    CALLS : NONE
//   IMPORT : aMenu, aOperator, cType
//   EXPORT : NONE
//-----------------------------------------------------------------------------

static procedure _OperMenu(aMenu, aOperator, cType)

  if valtype(cType) = 'C'
    if cType = 'L'
      aMenu     := { {"TRUE"}, {"FALSE"} }
      aOperator := { 'T', 'F' }
    endif
    if cType = 'N' .or. cType = 'D'
      aMenu     := { {"EQUAL TO"},    {"NOT EQUAL TO"},   {"LESS THAN"},      ;
                     {"GREATER THAN"},{"LESS THAN OR EQUAL TO"},              ;
                     {"GREATER OR EQUAL TO"} }
      aOperator := { '=', '<>', '<', '>', '<=', '>=' }
    endif
    if cType = 'C'
      aMenu     := { {"EQUAL TO"},    {"NOT EQUAL TO"},   {"LESS THAN"},      ;
                     {"GREATER THAN"},{"LESS THAN OR EQUAL TO"},              ;
                     {"GREATER OR EAUAL TO"}, {"CONTAINS"},                   ;
                     {"DOES NOT CONTAIN"} }
      aOperator := { '=', '<>', '<', '>', '<=', '>=', '$', '!$' }
    endif
  else
    aMenu     := {}
    aOperator := {}
  endif
  return


//-----------------------------------------------------------------------------
// FUNCTION : _DBFVIEW()
//  PURPOSE : Select database to View/Edit
//   IMPORT : NONE
//   CALLED : _SFCW()
//    CALLS : SelDbf(), _SFCVIEW()
//   EXPORT : NONE
//-----------------------------------------------------------------------------

static procedure _DbfView()

  store "" to csDbe, csIDX, csMEM

  csFileSpec := alltrim( upper( _SelDbf( csPath ) ) )
  csPath     := substr(csFileSpec, 1,                 rat('\', csFileSpec))
  csDBF      := substr(csFileSpec, rat('\', csFileSpec)+1, len(csFileSpec))
  csDBF      := substr(csDBF,      1,                 rat('.', csDBF)-1)
  //
  dbesetdefault( "FOXCDX" )
  csDbe   := "FOXCDX"
  //
  if file( csPath + csDBF + ".FPT" ) .or. file( csPath + csDBF + ".CDX" )
    //-
    //-- Found FOXPRO file formats
    //-
    csIDX := ".CDX"
    csMEM := ".FPT"
  elseif file( csPath + csDBF + ".DBT" ) .or. file( csPath + csDBF + ".NTX" )
    //-
    //-- Found DBASEII file formats
    //-
    dbesetdefault( "DBFNTX" )
    csDbe := "DBFNTX"
    csIDX := ".NTX"
    csMEM := ".DBT"
  endif
  _SfcView()
  return


//-----------------------------------------------------------------------------
// FUNCTION : _SELDBF()
//  PURPOSE : Select Drive and directory for database files
//   CALLED :
//    CALLS : NONE
//   IMPORT : NONE
//   EXPORT : FileSpec
//   UPDATE :  7 JUL 09
//-----------------------------------------------------------------------------

static function _SelDbf( cPath )

  local getlist := {}, GetOptions, lOk := .F., cFileName := space(60)
  local cTitle := "SELECTION STARTING @ "

  DEFAULT cPath := DC_SetDefault()
  cTitle        += cPath
  //
  @01,01 DCSAY "STARTING AT @ " + trim( cPath ) SAYSIZE 60 COLOR 2
  @02,01 DCGET cFileName POPUP {|c| DC_PopFile(NIL, cPath, "*.DBF", ;
      "STARTING PATH @" + cPath ) }
  //
  DCGETOPTIONS NOMINBUTTON NOMAXBUTTON NORESIZE EVAL {|o| setappwindow(o) }
  //
  DCREAD GUI TITLE cTitle FIT OPTIONS GetOptions ADDBUTTONS TO lOk
  //
  return iif(lOk, cFileName, "")


//-----------------------------------------------------------------------------
// FUNCTION : FINDIDX()
//  PURPOSE : Find available indexs
//   IMPORT : cDbf
//            cPath
//            cDbe
//   EXPORT : NIL
//-----------------------------------------------------------------------------

static function FindIdx(cDbf, cPath, cDbe)

  local cIdx

  DC_SetDefault(cPath)
  do case
    case cDbe == "DBFNTX"
      // Search required
    case cDbe == "FOXCDX"
      cIdx := cDbf + ".CDX"
      if file(cIdx)
        set index to cDbf
      endif
  endcase
  DC_SetDefault()
  return NIL


//-----------------------------------------------------------------------------
// FUNCTION : APPSYS()
//  PURPOSE : REQUIRED
//   CALLED :
//    CALLS : NONE
//   IMPORT : NONE
//   EXPORT : NONE
//-----------------------------------------------------------------------------

procedure AppSys()

  local aPos[2], aSize, nWidth := 800, nHeight := 600

  aSize           := setappwindow():currentsize()
  aPos[1]         := int((aSize[1]-nWidth) / 2)
  aPos[2]         := int((aSize[2]-nHeight) / 2)
  osDlg           := XBPDialog():new()
  osDlg:Title     := "SFCW - Simple Database Viewer"
  osDlg:MaxButton := .T.
  osDlg:MinButton := .T.
  osDlg:TaskList  := .T.
  osDlg:border    := XBPDLG_THINBORDER
  osDlg:create(,, aPos, { nWidth, nHeight },, .F.)
  osDlg:show()
  setappwindow( osDlg )
  setappfocus( osDlg )
  return


//-----------------------------------------------------------------------------
// FUNCTION : APPQUIT()
//  PURPOSE : REQUIRED
//   CALLED :
//    CALLS : NONE
//   IMPORT : NONE
//   EXPORT : NONE
//-----------------------------------------------------------------------------

procedure AppQuit()
  return


//-----------------------------------------------------------------------------
// FUNCTION : DLGHANDLER()
//  PURPOSE : HANDLE EVENTS
//   CALLED : _SFCVIEW()
//    CALLS : NONE
//   IMPORT : nEvent, mp1, mp2, oXbp, oDialog, getlist, aApp
//   EXPORT : IDX ORDER
//   UPDATE : 23 MAR 09
//-----------------------------------------------------------------------------

function DlgHandler(nEvent,mp1,mp2,oXbp, oDialog, getlist, aApp)

  if nEvent = xbeP_Close // .and. lMDIApp
    return DCGUI_EXIT_OK
  elseif valtype(oDialog:cargo) = 'L' .and. !oDialog:cargo // .and. !lMDIApp
    PostAppEvent( xbeP_Close )
  endif
  return DCGUI_NONE


//-----------------------------------------------------------------------------
// FUNCTION : _LOADDES()
//  PURPOSE : LOAD DATABASE ENGINES
//   CALLED : _SFCVIEW()
//    CALLS : NONE
//   IMPORT : NONE
//   EXPORT : IDX ORDER
//   UPDATE : 23 MAR 09
//-----------------------------------------------------------------------------

function _LoadDes()

  local i, aDbeList := DbeList(), cDbeList := ''

  if valtype(aDbeList) = 'A'
    for i := 1 to len(aDbeList)
      cDbeList += aDbeList[i,1] + ','
    next
  endif
  aDbeList := cDbeList
  if !('DBFDBE' $ aDbeList) .AND. !DbeLoad( "DBFDBE",.T.)
    DC_WinAlert( "Database-Engine DBFDBE not loaded" )
  endif
  if !('NTXDBE' $ aDbeList) .AND. !DbeLoad( "NTXDBE",.T.)
    DC_WinAlert( "Database-Engine NTXDBE not loaded"  )
  endif
  if !('DBFNTX' $ aDbeList) .AND. !DbeBuild( "DBFNTX", "DBFDBE", "NTXDBE" )
    DC_WinAlert( "DBFNTX Database-Engine, Could not build engine" )
  endif
  if !('CDXDBE' $ aDbeList) .AND. !DbeLoad( "CDXDBE",.T.)
    DC_WinAlert( "Database-Engine CDXDBE not loaded" )
  endif
  if !('DBFCDX' $ aDbeList) .AND. !DbeBuild( "DBFCDX", "DBFDBE", "CDXDBE" )
    DC_WinAlert( "DBFCDX Database-Engine, Could not build engine" )
  endif
  if !('FOXDBE' $ aDbeList) .AND. !DbeLoad( "FOXDBE",.T.)
    DC_WinAlert( "Database-Engine FOXDBE not loaded" )
  endif
  if !('FOXCDX' $ aDbeList) .AND. !DbeBuild( "FOXCDX", "FOXDBE", "CDXDBE" )
    DC_WinAlert( "FOXCDX Database-Engine, Could not build engine" )
  endif
  if !('DELDBE' $ aDbeList) .AND. !DbeLoad( "DELDBE",.T.)
    DC_WinAlert( "Database-Engine DELDBE not loaded" )
  endif
  return .T.


